<?php

abstract class Maarch_AbstractTestsSniff 
    implements PHP_CodeSniffer_Sniff
{
    protected $_phpcsFile = null;
    
    public function checkSpaceAfter($pter, $expectedLen=1)
    {
        $tokens = $this->_phpcsFile->getTokens();
        $tokenContent = $tokens[$pter]['content'];
        $spaceTokenPtr = $pter + 1;
        $spaceToken = $tokens[$spaceTokenPtr];
        
        if (!$this->isLastOnLine($pter)) {
            if ($spaceToken['code'] !== T_WHITESPACE) {
                $error = 'Expected %s space(s) after "%s"; found 0.';
                $data = array($expectedLen, $tokenContent);
                $this->_phpcsFile->addError($error, $spaceTokenPtr, 
                					 		'NoSpaceAfterToken', $data);
                return;
            } 
            $spaceContent = $spaceToken['content'];
            if (strlen($spaceContent) !== $expectedLen) {
                $error = 'Expected %s space(s) after "%s"; found %s.';
                $data = array($expectedLen, 
                              $tokenContent, 
                              strlen($spaceContent));
                $this->_phpcsFile->addError($error, $spaceTokenPtr, 
                							'SingleSpace', $data);
            } 
        }
        $this->checkLineBreakAfter($pter, false);
    }
    
    public function checkSpaceBefore($pter, $expectedLen=1)
    {
        $tokens = $this->_phpcsFile->getTokens();
        $tokenContent = $tokens[$pter]['content'];
        $spaceTokenPtr = $pter - 1;
        $spaceToken = $tokens[$spaceTokenPtr];
        if (!$this->isFirstOnLine($pter)) {
            if ($spaceToken['code'] !== T_WHITESPACE) {
                $error = 'Expected %s space(s) before "%s"; found 0';
                $data = array($expectedLen, $tokenContent);
                $this->_phpcsFile->addError($error, $pter, 
                					 		'NoSpaceBeforeToken', $data);
                return;
            } 
            $spaceContent = $spaceToken['content'];
            if (strlen($spaceContent) !== $expectedLen) {
                $error = 'Expected %s space(s) before "%s"; found %s.';
                $data = array($expectedLen, 
                              $tokenContent, 
                              strlen($spaceContent));
                $this->_phpcsFile->addError($error, $spaceTokenPtr, 
                							'SingleSpace', $data);
                return;
            }
        }
        $this->checkLineBreakBefore($pter, false);
    }
    
    public function checkLineBreakBefore($pter, $required=true)
    {
        $tokens = $this->_phpcsFile->getTokens();
        $tokenContent = $tokens[$pter]['content'];
        $prevToken = $this->_phpcsFile->findPrevious(
            array(T_WHITESPACE, T_COMMENT), 
            $pter - 1, max($pter - 20, 0), true
        );
        $isSameLine = ($tokens[$prevToken]['line'] === $tokens[$pter]['line']);
        if ($prevToken !== false) {
            
            if ($required && $isSameLine) {
                $error = 'There must be a line break between "%s" and "%s".';
                $data = array($tokens[$prevToken]['content'], $tokenContent);
                $this->_phpcsFile->addError($error, $pter, 
                							'NoLineBreak', $data);
            } elseif (!$required && !$isSameLine) {
                $error = 'There must be no line break between "%s" and "%s".';
                $data = array($tokens[$prevToken]['content'], $tokenContent);
                $this->_phpcsFile->addError($error, $pter, 
                							'LineBreak', $data);
            }
        }
    }
    
    public function checkLineBreakAfter($pter, $required=true)
    {
        $tokens = $this->_phpcsFile->getTokens();
        $tokenContent = $tokens[$pter]['content'];
        $nextToken = $this->_phpcsFile->findNext(
            array(T_WHITESPACE, T_COMMENT), 
            $pter + 1, $pter + 20, true
        );
        $isSameLine = ($tokens[$nextToken]['line'] === $tokens[$pter]['line']);
        if ($nextToken !== false) {
            if ($required && $isSameLine) {
                $error = 'There must be a line break between "%s" and "%s".';
                $data = array($tokenContent, $tokens[$nextToken]['content']);
                $this->_phpcsFile->addError($error, $nextToken, 
                							'NoLineBreak', $data);
            } elseif (!$required && !$isSameLine) {
                $error = 'There must be no line break between "%s" and "%s".';
                $data = array($tokenContent, $tokens[$nextToken]['content']);
                $this->_phpcsFile->addError($error, $nextToken, 
                							'LineBreak', $data);
            }
        }
    }
    
    public function isFirstOnLine($pter)
    {
        $firstOnLine = $this->_phpcsFile->findFirstOnLine(
            array(T_WHITESPACE, T_COMMENT),
            $pter, true
        );
        return ($pter === $firstOnLine);
    }
    
    public function isLastOnLine($pter)
    {
        $tokens = $this->_phpcsFile->getTokens();
        $nextToken = $this->_phpcsFile->findNext(
            array(T_WHITESPACE, T_COMMENT),
            $pter + 1, $pter + 20, true
        );
        return ($tokens[$pter]['line'] !== $tokens[$nextToken]['line']);
    }
}
