<?php

class Maarch_Sniffs_Arrays_ArrayDeclarationSniff 
    implements PHP_CodeSniffer_Sniff
{

    public function register()
    {
        return array(T_ARRAY);

    }//end register()


    /**
     * Processes this sniff, when one of its tokens is encountered.
     *
     * @param PHP_CodeSniffer_File $phpcsFile The current file being checked.
     * @param int $stackPtr  The position of the current token in the
     *                                        stack passed in $tokens.
     *
     * @return void
     */
    public function process(PHP_CodeSniffer_File $phpcsFile, $stackPtr)
    {
        
        $tokens = $phpcsFile->getTokens();
        
        $arrayStart   = $tokens[$stackPtr]['parenthesis_opener'];
        $arrayEnd     = $tokens[$arrayStart]['parenthesis_closer'];
        $keywordStart = $tokens[$stackPtr]['column'];
        $arrayNestingLevel = array_key_exists('nested_parenthesis', 
                                              $tokens[$stackPtr])
                        ? (count($tokens[$stackPtr]['nested_parenthesis']) + 1)
                        : 1;
        $arrayStartLine = $tokens[$arrayStart]['line'];
        $arrayEndLine = $tokens[$arrayEnd]['line'];
        $isSingleLine = ($arrayStartLine === $arrayEndLine);
        $isAssociative = (bool) $phpcsFile->findNext(array(T_DOUBLE_ARROW), 
                                                     $arrayStart, $arrayEnd);
        $firstToken = $phpcsFile->findNext(
            array(T_WHITESPACE, T_COMMENT), 
            ($arrayStart + 1), 
            ($arrayEnd + 1), 
            true
        );
        $lastToken = $phpcsFile->findPrevious(
            array(T_WHITESPACE, T_COMMENT), 
            ($arrayEnd - 1), 
            $arrayStart, 
            true
        );
        

        if ($arrayStart != ($stackPtr + 1)) {
            $error = 'There must be no space between the Array keyword '
                   . 'and the opening parenthesis';
            $phpcsFile->addError($error, $stackPtr, 'SpaceAfterKeyword');
        }

        // Check for empty arrays.
        if ($firstToken === $arrayEnd) {
            // Empty array, but if the brackets aren't together, 
            // there's a problem.
            if (($arrayEnd - $arrayStart) !== 1) {
                $error = 'Empty array declaration must have no space' 
                       . 'between the parentheses';
                $phpcsFile->addError($error, $stackPtr, 'SpaceInEmptyArray');

                // We can return here because there is nothing else to check. 
                // All code below can assume that the array is not empty.
                return;
            }
        }
        
        //check for spaces after commas
        $nextComma  = $arrayStart;
        while ( 
            ($nextComma = $phpcsFile->findNext(
                array(T_COMMA), 
                ($nextComma + 1), 
                $arrayEnd
            )) !== false
        ) {
            $nextToken = $phpcsFile->findNext(
                array(T_WHITESPACE, T_COMMENT), 
                $nextComma + 1,
                $arrayEnd,
                true
            );
            if ($tokens[$nextToken]['line'] !== $tokens[$nextComma]['line']) {
                $isEndOfLine = true;
            } else {
                $isEndOfLine = false;
            }
            
            if (!$isEndOfLine) {
                //check if there is one space after each comma
                if ($tokens[$nextComma + 1]['code'] !== T_WHITESPACE) {
                    $content = $tokens[($nextComma + 1)]['content'];
                    $error = 'Expected 1 space between comma and "%s"; 0 found';
                    $data = array($content);
                    $phpcsFile->addError($error, $nextComma, 
                                         'NoSpaceAfterComma', $data);
                } else {
                    $spaceLength = strlen($tokens[$nextComma + 1]['content']); 
                    if ($spaceLength !== 1) {
                        $content = $tokens[($nextComma + 2)]['content'];
                        $error = 'Expected 1 space between comma and "%s"; '
                               . '%s found';
                        $data = array($content, $spaceLength);
                        $phpcsFile->addError($error, $nextComma, 
                                             'SpaceAfterComma', $data);
                    }
                }
                // arrays must have each pair on its own line
                if ($isAssociative) {
                    $nextArrow = $phpcsFile->findNext(
                        array(T_DOUBLE_ARROW),
                        $nextComma + 1,
                        $arrayEnd
                    ); 
                    if ($tokens[$nextArrow]['line'] === $tokens[$nextComma]['line']) {
                        $error = 'Associatives arrays must have each pair '
                           . 'on its own line';
                        $phpcsFile->addError($error, $nextArrow, 
                    					     'OnePairPerLine');
                    }
                }
            }
            
        }
        
        if ($isSingleLine) {
            //There must be no space before the closing parenthesis
            if ($tokens[$lastToken]['code'] === T_COMMA) {
                $error   = 'Last item must not be followed by a comma';
                $data    = array();
                $phpcsFile->addError($error, $lastToken, 
                    				 'NoCommaBeforeBracket', $data);
                return;
            } elseif ($lastToken !== ($arrayEnd - 1)) {
                $error   = 'There must be no space before closing parentheses';
                $data    = array();
                $phpcsFile->addError($error, $lastToken, 
                                     'NoSpaceBeforeClosingParentheses', $data);
            }
            //There must be no space after the opening parenthesis
            if ($firstToken !== ($arrayStart + 1)) {
                $error   = 'There must be no space after opening parentheses';
                $data    = array();
                $phpcsFile->addError($error, $firstToken, 
                                     'NoSpaceAfterOpeningParentheses', $data);
            }
        
              
        } else {
            // end singleline checks, start multiline
            
            $startUnderArray = ($tokens[$firstToken]['line'] !== $arrayStartLine);
            $parenOnNewLine = ($arrayEndLine !== $tokens[$lastToken]['line']);
            $lastTokenIsComma = ($tokens[$lastToken]['code'] === T_COMMA);
            
            if ($startUnderArray) {
                if (!$parenOnNewLine) {
                    $error = 'Closing parenthesis of array declaration must be '
                           . 'on a new line';
                    $phpcsFile->addError($error, $arrayEnd, 
                                     	 'CloseBraceNewLine');
                }
                if (!$lastTokenIsComma) {
                    $warning = 'There should be a comma after '
                        	 . 'the last element';
                    $phpcsFile->addWarning($warning, $lastToken, 
                                       	   'NoCommaLastItem');
                }
            } else {
                if ($parenOnNewLine) {
                    // The closing parenthesis must be on the last line
                    $error = 'The closing parenthesis must be on the '
                           . 'last line.';
                    $phpcsFile->addError($error, $arrayEnd, 
                                         'CloseBraceNewLine');
                }
                if ($lastTokenIsComma) {
                    $error = 'There should be no comma after the last element';
                    $phpcsFile->addError($error, $lastToken, 'CommaLastItem');
                }
            }
            
            
            //
            // Indentation check
            //
            if ($startUnderArray) {
                $firstItemOnDeclLine = $phpcsFile->findFirstOnLine(
                    array(T_WHITESPACE), 
                    $arrayStart, 
                    true
                );
                $expectedIndent = $tokens[$firstItemOnDeclLine]['column'] + 4;
            } else {
                $expectedIndent = $tokens[$firstToken]['column'];
            }
            
            $currentLine = $tokens[$arrayStart]['line'];
            $currentTokenPtr = $arrayStart;
            while (
                $currentTokenPtr = $phpcsFile->findNext(
                    array(T_WHITESPACE), 
                    $currentTokenPtr + 1, 
                    $arrayEnd - 1, 
                    true
                )
            ) {
                $currentToken = $tokens[$currentTokenPtr];
                //if it is an array in an array, we pass checks
                $curNestingLevel = count($currentToken['nested_parenthesis']);
                if ($curNestingLevel !== $arrayNestingLevel) {
                    $currentLine = $currentToken['line'];
                    continue;
                }
                
                // To verify indentation, we only care about first 
                // token on the line
                if ($currentToken['line'] === $currentLine) {
                    continue;
                }
                
                // here is the actual check
                $currentLine = $currentToken['line'];
                if ($currentToken['column'] !== $expectedIndent) {
                    $data = array(
                        $expectedIndent - 1, 
                        $currentToken['column'] - 1,
                    );
                    $error = 'Bad indent; expected %s, found %s.';
                    $phpcsFile->addError($error, $currentTokenPtr, 
                                         "BadIndent", $data);
                }
            }
        }
    }//end process()


}//end class

