<?php

require_once __DIR__ . '/../../AbstractTestsSniff.php';

class Maarch_Sniffs_ControlStructures_ControlStructureSignatureSniff 
    extends Maarch_AbstractTestsSniff
{
    
    private $_hasCondition = array(
        T_IF => true,
        T_DO => false,
        T_WHILE => true,
        T_FOR => true,
        T_FOREACH => true,
        T_ELSE => false,
        T_ELSEIF => true,
    );
    
    public function register()
    {
        return array(
            T_IF, 
            T_DO,
            T_WHILE,
            T_FOR,
            T_FOREACH,
            T_ELSE,
            T_ELSEIF,
        );
    }
    
    
    /*
     * do {EOL...} while (...);EOL
     * while (...) {EOL
     * for (...) {EOL
     * if (...) {EOL
     * foreach (...) {EOL
     * } else if (...) {EOL
     * } elseif (...) {EOL
     * } else {EOL
     * do {EOL
     */
    public function process(PHP_CodeSniffer_File $phpcsFile, $stackPtr) 
    {
        $this->_phpcsFile = $phpcsFile;
        $tokens = $phpcsFile->getTokens();
        
        $scPtr = $stackPtr;
        $scToken = $tokens[$scPtr];
        $scName = $scToken['content'];
        
        $passBracketsCheck = false;
        if ($scToken['code'] === T_WHILE) {
            $isWhileAfterDo = $this::_isWhileAfterDo($scPtr);
            if ($isWhileAfterDo) {
                $passBracketsCheck = true;
            }
        } elseif ($scToken['code'] === T_ELSE) {
            $isElseBeforeIf = ($tokens[$scPtr + 2]['code'] === T_IF);
            if ($isElseBeforeIf) {
                $passBracketsCheck = true;
            }
        }
        
        /*
         * Check spacing
         */
        
        //check for space after keyword
        $this->checkSpaceAfter($scPtr);
        
        // check space before "else", "elseif" and "while" 
        // in "do {...} while" constructs
        $passBreakLineAfterPreviousBracket = false;
        if (($scToken['code'] === T_WHILE && $this->_isWhileAfterDo($scPtr)) 
            || in_array($scToken['code'], array(T_ELSE, T_ELSEIF))
        ) {
            $this->checkSpaceBefore($scPtr);
            $passBreakLineAfterPreviousBracket = true;
        }
        
        /*
         * Check Brackets Position
         */
        if (!$passBracketsCheck) {
            // check for brackets
            if (array_key_exists('scope_opener', $scToken)) {
                $openBracket = $scToken['scope_opener'];
            } else {
                $error = 'Brackets must not be omitted';
                $phpcsFile->addError($error, $scPtr, 'NoBrackets');
                return;
            }
            if (array_key_exists('scope_closer', $scToken)) {
                $closeBracket = $scToken['scope_closer'];
            } else {
                $error = 'Missing closing bracket';
                $phpcsFile->addError($error, $scPtr, 'NoClosingBrackets');
            }
            
            // check bracket line
            if ($this->_hasCondition[$scToken['code']] === true) {
                $openParen = $scToken['parenthesis_opener'];
                $openParenLine = $tokens[$openParen]['line'];
                $closeParen = $scToken['parenthesis_closer'];
                $closeParenLine = $tokens[$closeParen]['line'];
                $isMultiLine = ($openParenLine !== $closeParenLine);
                //$isMultiLine = ($openBracketLine !== $scToken['line']);
                
                $this->checkLineBreakBefore($closeParen, $isMultiLine);
            }
            
            // check for a space before opening bracket
            if ($this->_hasCondition[$scToken['code']] === true) {
                $this->checkSpaceBefore($openBracket);
            }
            
            $this->checkLineBreakAfter($openBracket);
            $this->checkLineBreakBefore($closeBracket);
            
            
            // check for line break after close bracket, except if it is 
            // followed by "else", "elseif" or "while" 
            $tokenAfterClosingBracket = $phpcsFile->findNext(
                array(T_WHITESPACE), $closeBracket + 1, null, true
            );
            $tokensToIgnore = array(T_ELSE, T_ELSEIF, T_WHILE);
            if ($tokenAfterClosingBracket !== false 
                && !in_array($tokens[$tokenAfterClosingBracket]['code'], 
                             $tokensToIgnore)
            ) {
                $this->checkLineBreakAfter($closeBracket);
            }
        } // end of if !$passBracketsCheck 
    } //end process
    
    private function _isWhileAfterDo($scPtr)
    {
        $tokens = $this->_phpcsFile->getTokens();
        $previousBracket = $this->_phpcsFile->findPrevious(
            array(T_CLOSE_CURLY_BRACKET, T_SEMICOLON), $scPtr, 0
        );
        if ($previousBracket !== false 
            && $tokens[$previousBracket]['code'] === T_CLOSE_CURLY_BRACKET
        ) {
            $bracketOwner = $tokens[$previousBracket]['scope_condition'];
            if ($tokens[$bracketOwner]['code'] === T_DO) {
                return true;
            }
        }
        return false;
    }
}